#include<iostream>
#include<fstream>
#include<stdlib.h>
#include<vector>
#include<cmath>
#include<string>
#include<signal.h>

/* This program solves D variables (D mod 6=0), N dense constraints problem by 
 * projected subgradient descent.
 *
 * where (1) each variable is constrained to be non-negative, and
 * 	 (2) x(6k)+...+x(6k+5) <= 1, k=0~(D/6)
 * */


using namespace std;

int D = 61206;
int N = 304;
//int D = 242406;
//int N = 604;

double** A; //N by D
double* a_sq;
double* b; //N by 1
double* c; //D by 1
double beta = 1.0;

char* file_x_out = "x_out.txt";
double* x = new double[D];
double* x_best = new double[D];

void onTerminate(int param){
	cout << "output solution..." << endl;
	ofstream fout(file_x_out);
	for(int i=0;i<D;i++){

		fout << x_best[i] << endl;
	}
	fout.close();
	exit(1);
}

//Shrinking Pamameters
int K=50;
double thd=1.0;

void readMatrix(double** A, char* fileName, int R, int C){
	
	ifstream fin(fileName);
	for(int i=0;i<R;i++){
		for(int j=0;j<C;j++){
			fin >> A[i][j];
		}
	}
	fin.close();
}

void readMatrixFIFO(double** A, char* fifoName, int R, int C){
	
	ifstream fin(fifoName,ios::binary|ios::in);
	int size;
	fin.read((char*)&size,sizeof(int));
	
	double* arr = new double[size];
	fin.read( (char*) arr, size*sizeof(double) );
	
	for(int i=0;i<R;i++){
		for(int j=0;j<C;j++){
			
			A[i][j] = arr[i*C+j];
		}
	}
	
	fin.close();
	
	delete arr;
}

void readVector(double* v, char* fileName, int R){
	
	ifstream fin(fileName);
	for(int i=0;i<R;i++){
		fin >> v[i];
	}
	fin.close();
}

double dot( double* a, double* b, int R ){
	
	double sum=0.0;
	for(int i=0;i<R;i++){
		sum += a[i]*b[i];
	}
	
	return sum;
}

int nnz( double* x , int size ){
	
	int c=0;
	for(int i=0;i<size;i++)
		if( x[i] > 1e-3 )
			c++;

	return c;
}

void project2( double* x ){

	//x(6k)+...x(6k+5) <= 1, k=0~D/6
	int i,j;
	double sum;
	for(int k=0;k<(D/6);k++){
		
		i = 6*k;
		
		sum=0.0;
		for(int j=i;j<i+6;j++)
			sum += x[j];

		if(sum>1.0){

			for(int j=i; j<i+6; j++)
				x[j] /= sum;
		}
	}
}

void project( double* x ){

	//x(6k)+...x(6k+5) <= 1, k=0~D/6-1
	int i,j;
	double sum;
	int nnz;
	int nnz_index[6];
	for(int k=0;k<(D/6);k++){
		
		i = 6*k;
		
		nnz = 6;
		for(int j=0;j<nnz;j++)
			nnz_index[j] = j;
		
		sum=0.0;
		for(int j=0;j<nnz;j++)
			sum += x[i+nnz_index[j]];
		double cof = (sum - 1)/nnz;

		while( sum > 1.0 ){
			
			//line search
			double min=cof;
			int argmin=-1;
			for(int j=0;j<nnz;j++)
				if( x[ i+nnz_index[j] ] < min ){
					min = x[ i+nnz_index[j] ];
					argmin = j;
				}

			//hit argmin
			if( argmin == -1 ){
				for(int j=0;j<nnz;j++){
					x[ i+nnz_index[j] ] -= cof;
				}
				break;
			}else{
				for(int j=0;j<nnz;j++)
					x[ i+nnz_index[j] ] -= x[ i+nnz_index[argmin] ];
				
				//project on argmin
				swap(nnz_index[argmin],nnz_index[nnz-1]);
				nnz--;

				//compute cof
				sum=0.0;
				for(int j=0;j<nnz;j++)
					sum += x[i+nnz_index[j]];
				cof = (sum-1)/nnz;
			}
		}
	}
}


int sparse_subgrad( double* x , double eta ){
	
	int i,j;
	double sum;
	int count=0;
	for(int k=0;k<(D/6);k++){
		
		i = 6*k;
		
		sum=0.0;
		for(int j=i;j<i+6;j++)
			sum += x[j];

		if(sum>1.0){ //violated
			count++;
			for(int k=0;k<D;k++){
				x[k] = max( x[k] - eta*1.0, 0.0 );
			}
		}
	}
	
	return count;
}

template<class T>
void shuffle( vector<T>& vect ){

	int r;
	for(int i=0;i<vect.size()-1;i++){
		r =  (rand() % (vect.size() - i-1))+1+i ;
		swap(vect[i],vect[r]);
	}
}

int check(double* x){
	
	//check Ax-b <= 0
	double* a;
	double prod;
	for(int i=0;i<N;i++){
		a = A[i];
		prod = dot(a,x,D);
		if( prod - b[i] > 0 ){
			cout << "ax-b=" << prod-b[i] << endl;
			return 1;
		}
	}

	//check sparse constraint
	
	int i,j;
	for(int k=0;k<D/6;k++){

		i = k*6;
		double sum=0.0;
		for(int j=i;j<i+6;j++)
			sum += x[j];

		if( sum > 1.0 + 1e-4 ){
			cout << "sum=" << sum << endl;
			return 2;
		}
	}
	
	//check non-negativity
	for(int k=0;k<D;k++)
		if( x[k] < 0 )
			return 3;
	return 0;
}

int main(int argc,char** argv){
	
	if( argc < 6 ){
		cerr << "Usage: SubLP [A_dense] [b_dense] [c] [iter] [lambda] [eta0] (x_init)" << endl;
		exit(0);
	}
	srand(time(NULL));

	char* file_A = argv[1];
	char* file_b = argv[2];
	//double b_end = atof(argv[2]);
	char* file_c = argv[3];
	int iter = atoi(argv[4]);
	double lambda = atoi(argv[5]);
	double eta0 = atoi(argv[6]);
	
	//initialize
	for(int i=0;i<D;i++){
		x[i] = 0;
		x_best[i] = 0;
	}

	if( argc >= 8 ){
		
		char* file_x_init = argv[7];
		ifstream fin(file_x_init);
		for(int i=0;i<D;i++){
			fin >> x[i] ;
			x_best[i] = x[i];
		}
		fin.close();
	}
	
	//read A,b,c
	A = new double*[N];
	for(int i=0;i<N;i++)
		A[i] = new double[D];
	
	string str(file_A);
	int index = str.find_last_of("\.");
	string suffix = str.substr(index+1,str.size());
	
	if(suffix!="fifo"){
		readMatrix(A,file_A,N,D);
	}else{
		readMatrixFIFO(A,file_A,N,D);
	}
	
	b = new double[N];
	readVector(b,file_b,N);
	//for(int i=0;i<N-1;i++){
	//	b[i] = 0.05;
	//}
	//b[N-1] = b_end;
	
	c = new double[D];
	readVector(c,file_c,D);
	
	//Main Algorithm
	double eta = eta0;
	
	
	int r;
	double* a;
	double br, a_x;
	double prod;
	double fmax=0.0;
	vector<int> indexes;
	for(int i=0;i<N;i++)
		indexes.push_back(i);
	shuffle(indexes);
	int violate;
	int min_violate=1000;
	bool shrinking=true;
	
	double* smooth_grad = new double[D];
	for(int i=0;i<D;i++)
		smooth_grad[i] = 0.0;
	
	int* shrink = new int[N];
	for(int i=0;i<N;i++){
		shrink[i] = 0;
	}
	
	void (*prev_fn)(int);

	signal (SIGINT,onTerminate);

	for(int i=0;i<iter;i++){
		
		//eta = 1.0/sqrt((double)i+1);
		eta = eta0/pow((double)(i+1),1/3.0);
		
		violate=0;
		double max_ax_b = -1.0;
		for(int j=0;j<N;j++){
			
			r = indexes[j];
			
			if( shrinking && shrink[r] > 0 ){
				shrink[r]--;
				continue;
			}
			
			a = A[r];
			a_x = dot(a,x,D);
			if( a_x - b[r] > 0 ){
				
				violate++;
				shrinking = true;
				if( a_x - b[r] > max_ax_b ){
					max_ax_b = a_x - b[r];
				}
				
				
				for(int k=0;k<D;k++)
					x[k] = max( x[k] - eta*a[k] , 0.0 );
				
				/*
				for(int k=0;k<D;k++){
					smooth_grad[k] = (beta*(-a[k])+(1-beta)*smooth_grad[k]);
					x[k] = max( x[k]+eta*smooth_grad[k], 0.0 );
				}
				*/
				
				
			}else{
				//cout << "a_x-b=" << a_x-b[r] << " thd=" << thd << endl;
				
				if( shrinking && (a_x - b[r]  <  -thd*eta) ){
					
					shrink[r] = K;
				}
				
			}
		}
		//sub-gradient on sparse constraints
		//violate += (sparse_subgrad( x, 1.0/sqrt((double)i+1) )>0);
		
		if( max_ax_b != -1.0 )
			thd = max_ax_b;
		
		if( violate == 0 && shrinking ){
			for(int j=0;j<N;j++)
				shrink[j] = 0;
			shrinking=false;
			continue;
		}
		
		
		if( violate == 0 ){
			
			int check_result = check(x);
			cout << "check=" << check_result  << endl;
			double obj = dot(c,x,D);
			if( obj > fmax && check_result==0){
				fmax = obj;
				cout << "fmax=" << fmax  << endl;
				
				for(int i=0;i<D;i++)
					x_best[i] = x[i];
			}
			
			//for(int j=0;j<N;j++){
				
			//	project(x);
				
				for(int k=0;k<D;k++){
					/*
					smooth_grad[k] = (beta*c[k]*lambda + (1-beta)*smooth_grad[k]);
					x[k] = max( x[k] + eta*smooth_grad[k], 0.0 );
					*/

					x[k] = max( x[k] + eta*lambda*c[k] , 0.0 );
				}
			//}
		}
		project(x);
		shuffle(indexes);
	}
}
