## Input values for discretized optimization problem

# If running parallel computations using different ncp_alt values, set here: 
ncp_alt_inflation_factor <- 1 # default to no inflation factor
# ncp_alt_inflation_factor_list <- seq(1,1.1,length=10)
# ncp_alt_inflation_factor <- ncp_alt_inflation_factor_list[task_id] # task_id passed by function generate_... that call this function

max_error_prob <- 0
alpha <- (0.05 - 10^(-4)) ## desired FWER
beta <- (1-0.875) ## 1 minus beta is desired power to reject H0C at ncp_alt defined below
p1<- 0.5 # proportion of population in subpopulation 1
p2<- 1-p1 # proportion of population in subpopulation 2
rho1 <- sqrt(p1) # correlation of Z_1 and Z_C
rho2 <- sqrt(p2) # correlation of Z_2 and Z_C
ncp_alt <- ncp_alt_inflation_factor*(qnorm(1-alpha) + qnorm(0.9))*c(rho1,rho2) # alternative used in power constraint which is set so that UMP test of H0C alone has power 0.9 to reject, at level alpha. 
delta_1_min <- ncp_alt[1]-(10^(-10)) # set just below alternative used in power constraint, so as to be compatible with loss function and prior below
delta_2_min <- ncp_alt[2]-(10^(-10))
#version <- as.integer(Sys.getenv("SGE_TASK_ID"))
#set widths of large square outside of which multiple testing procedure rejects no null hypotheses (corresponds to w in text)
w1 <- 5  
w2 <- 5
# dimension of small squares used in discretization
tau <- 0.1

number_actions <- 6 # number of actions excluding default action 0
					# These correspond to rejecting the subsets of null hypotheses: 
					# S = (H01,H02,H0C,{H01,H0C},{H02,H0C},{H01,H02,H0C}) 

map_from_P_to_type_I_error_indicator_over_set_of_actions <- function(ncp)
{
	return(c(ncp[1]<=0,ncp[2]<=0,rho1*ncp[1]+rho2*ncp[2]<=0,ncp[1]<=0 || rho1*ncp[1]+rho2*ncp[2]<=0,ncp[2]<=0 || rho1*ncp[1]+rho2*ncp[2]<=0,ncp[1]<=0 || ncp[2]<=0 || rho1*ncp[1]+rho2*ncp[2]<=0))
}

indicator_contribute_to_combined_pop_power <- c(0,0,1,1,1,1)

# Loss function
L <- function(ncp){ # returns vector corresponding to loss function values at each action, for data generating distribution defined by non-centrality parameters ncp
	L1 <- as.numeric(ncp[1]>delta_1_min)*c(1,0,1,1,0,1,0) # indicator of delta_1 > delta_1_min and subset S_j (defined above) containing H_{01}
	L2 <- as.numeric(ncp[2]>delta_2_min)*c(1,1,0,1,1,0,0) # indicator of delta_2 > delta_2_min and subset S_j containing H_{02}
	L_total <- L1+L2
	return(L_total[2:7] - L_total[1]) # subtract L_total[1] since first element represents default option, which we do not allocate a variable for in our LP representation.
}

prior_weights <- c(1/4,1/4,1/4,1/4)
prior_support <- c(list(c(0,0)),list(c(ncp_alt[1],0)),list(c(0,ncp_alt[2])),list(ncp_alt))

# list of pairs of non-centrality parameters in G_{tau,w}
ncp_list <- list()
for(x1 in seq(-w1,w2,by=tau)) {ncp_list <- c(ncp_list,list(c(x1,0)))}
for(y in seq(-w1,w2,by=tau)) {ncp_list <- c(ncp_list,list(c(0,y)))}
for(z in seq(-w1/rho2,w2/rho1,by=tau)) {ncp_list <- c(ncp_list,list(c(rho2*z,-rho1*z)))}
for(z in c(-seq(1.07,1.16,by=0.005),seq(1.07,1.16,by=0.005))){ncp_list <- c(ncp_list,list(c(z,-(rho1/rho2)*z)))} ## These were selected based on a previous computation showing active constraints are probably in this region, so we are making grid finer here
ncp_list <- unique(ncp_list)

# Settings for integration region and precision in computing lower bound to original problem using dual solution to discretized problem
max_eval_iters <- 100000
w1_unconst <- 5
w2_unconst <- 5

# Set grid density for grid search used in computing worst-case familywise Type I error rate
# Each rectangle in grid set to have dimensions:
grid_width_1 <- 0.01
grid_width_2 <- 0.01

# settings for Bayes Risk vs. H0C power tradeoff plots, if producing this
filename_for_power_tradeoff_plots <- "power_tradeoff_plots.eps"
one_minus_beta_list <- c(seq(0.8,0.89,length=10),seq(0.891,0.899,length=9))
write(one_minus_beta_list,f="one_minus_beta_list.txt")


