#ifdef __cplusplus
extern "C" {
#endif

#include "sortsearch.h" 

/*  *** qksort: sort v[left]...v[right] into increasing order ***
        reference: The C Programming Language,
                   Kernighan and Ritchie
                   2nd. edition, page 87.  */

void qksort(
        double     *v,
        int     left,
        int     right
)
{
        int i, last;

        if (left >= right) return;   /* do nothing if array contains
                                        fewer than two elements */

        dswap(v, left, (left + right)/2);        /* move partition elem */
        last = left;                            /* to v[left] */
        for (i = left+1; i <= right; i++)       /* partition */
            if (v[i] < v[left])
                dswap(v, ++last, i);
        dswap(v, left, last);                    /* restore partition elem */
        qksort(v, left, last-1);
        qksort(v, last+1, right);
}

/*  *** dswap: interchange v[i] and v[j] */

void dswap(
        double     *v,
        int     i,
        int     j
)
{
        double temp;

        temp = v[i];
        v[i] = v[j];
        v[j] = temp;
}

#ifdef __cplusplus
}
#endif
